/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * This class implements the {@link ICommandStackService} interface. This class
 * mainly delegates its calls to the {@link IPersistencyService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: BA190E69F3975AEA8C17310496C653BE
 */
public class CommandStackService implements ICommandStackService {
	/** {@inheritDoc} */
	@Override
	public void runAsCommand(EObject target, Runnable runner) {
		ITopLevelElement context = IPersistencyService.INSTANCE
				.getTopLevelElementFor(target);
		if (context != null) {
			context.runAsCommand(runner);
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean canUndo(EObject target) {
		ITopLevelElement context = IPersistencyService.INSTANCE
				.getTopLevelElementFor(target);
		return context != null && context.canUndo();
	}

	/** {@inheritDoc} */
	@Override
	public boolean canRedo(EObject target) {
		ITopLevelElement context = IPersistencyService.INSTANCE
				.getTopLevelElementFor(target);
		return context != null && context.canRedo();
	}

	/** {@inheritDoc} */
	@Override
	public void undo(EObject target) {
		IPersistencyService.INSTANCE.getTopLevelElementFor(target).undo();

	}

	/** {@inheritDoc} */
	@Override
	public void redo(EObject target) {
		IPersistencyService.INSTANCE.getTopLevelElementFor(target).redo();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isDirty(EObject target) {
		ITopLevelElement context = IPersistencyService.INSTANCE
				.getTopLevelElementFor(target);
		return context != null && context.isDirty();
	}

	/** {@inheritDoc} */
	@Override
	public void doSave(EObject target, IProgressMonitor monitor) {
		ITopLevelElement context = IPersistencyService.INSTANCE
				.getTopLevelElementFor(target);
		if (context != null && context.isDirty()) {
			try {
				context.doSave(monitor);
			} catch (Exception e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Error while saving.", e);
			}
		}
	}
}
